@@ -1,6 +1,6 @@
 /* pvm.h - PVM values  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023, 2024 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,12 +60,12 @@
    Bits marked with `x' are unused and should be always 0.  */
 
 #define PVM_VAL_INT_SIZE(V) (((int) (((V) >> 3) & 0x1f)) + 1)
-#define PVM_VAL_INT(V) (((int32_t) ((V) >> 32))                \
-                        << (32 - PVM_VAL_INT_SIZE ((V)))       \
+#define PVM_VAL_INT(V) (((int32_t) (((uint32_t) ((V) >> 32))            \
+                                    << (32 - PVM_VAL_INT_SIZE ((V)))))  \
                         >> (32 - PVM_VAL_INT_SIZE ((V))))
-#define PVM_MAKE_INT(V,S)                       \
-  (((((int64_t) (V)) & 0xffffffff) << 32)       \
-   | ((((S) - 1) & 0x1f) << 3)                  \
+#define PVM_MAKE_INT(V,S)                            \
+  (((((uint64_t) (int64_t) (V)) & 0xffffffff) << 32) \
+   | ((uint32_t)(((S) - 1) & 0x1f) << 3)             \
    | PVM_VAL_TAG_INT)
 
 #define PVM_VAL_UINT_SIZE(V) (((int) (((V) >> 3) & 0x1f)) + 1)
@@ -73,7 +73,7 @@
                          & ((uint32_t) (~( ((~0ul) << ((PVM_VAL_UINT_SIZE ((V)))-1)) << 1 ))))
 #define PVM_MAKE_UINT(V,S)                      \
   (((((uint64_t) (V)) & 0xffffffff) << 32)      \
-   | ((((S) - 1) & 0x1f) << 3)                  \
+   | ((uint32_t) (((S) - 1) & 0x1f) << 3)       \
    | PVM_VAL_TAG_UINT)
 
 #define PVM_MAX_UINT(size) ((1U << (size)) - 1)
@@ -108,8 +108,8 @@
     ((uint64_t) (uintptr_t) ll) | (T); })
 
 #define PVM_VAL_LONG_SIZE(V) (_PVM_VAL_LONG_ULONG_SIZE (V))
-#define PVM_VAL_LONG(V) (_PVM_VAL_LONG_ULONG_VAL ((V))           \
-                         << (64 - PVM_VAL_LONG_SIZE ((V)))      \
+#define PVM_VAL_LONG(V) (((int64_t) ((uint64_t) _PVM_VAL_LONG_ULONG_VAL ((V)) \
+                                     << (64 - PVM_VAL_LONG_SIZE ((V))))) \
                          >> (64 - PVM_VAL_LONG_SIZE ((V))))
 #define PVM_MAKE_LONG(V,S)                              \
   (PVM_MAKE_LONG_ULONG ((V),(S),PVM_VAL_TAG_LONG))
@@ -204,9 +204,6 @@ struct pvm_mapinfo
   pvm_val offset;
 };
 
-struct pvm_mapinfo pvm_make_mapinfo (int mapped_p, pvm_val ios,
-                                     pvm_val offset);
-
 /* Arrays values are boxed, and store sequences of homogeneous values
    called array "elements".  They can be mapped in IO, or unmapped.
 
@@ -282,7 +279,7 @@ typedef struct pvm_array *pvm_array;
    how to obtain these values.
 
    OFFSET is an ulong<64> value holding the bit offset of the element,
-   relative to the begginnig of the IO space.  If the array is not
+   relative to the beginning of the IO space.  If the array is not
    mapped then this is PVM_NULL.
 
    OFFSET_BACK is a backup area used by the reloc instructions.
@@ -426,6 +423,7 @@ typedef struct pvm_struct *pvm_struct;
 #define PVM_VAL_TYP_A_BOUND(V) (PVM_VAL_TYP((V))->val.array.bound)
 #define PVM_VAL_TYP_A_ETYPE(V) (PVM_VAL_TYP((V))->val.array.etype)
 #define PVM_VAL_TYP_S_NAME(V) (PVM_VAL_TYP((V))->val.sct.name)
+#define PVM_VAL_TYP_S_CONSTRUCTOR(V) (PVM_VAL_TYP((V))->val.sct.constructor)
 #define PVM_VAL_TYP_S_NFIELDS(V) (PVM_VAL_TYP((V))->val.sct.nfields)
 #define PVM_VAL_TYP_S_FNAMES(V) (PVM_VAL_TYP((V))->val.sct.fnames)
 #define PVM_VAL_TYP_S_FTYPES(V) (PVM_VAL_TYP((V))->val.sct.ftypes)
@@ -433,6 +431,7 @@ typedef struct pvm_struct *pvm_struct;
 #define PVM_VAL_TYP_S_FTYPE(V,I) (PVM_VAL_TYP_S_FTYPES((V))[(I)])
 #define PVM_VAL_TYP_O_UNIT(V) (PVM_VAL_TYP((V))->val.off.unit)
 #define PVM_VAL_TYP_O_BASE_TYPE(V) (PVM_VAL_TYP((V))->val.off.base_type)
+#define PVM_VAL_TYP_O_REF_TYPE(V) (PVM_VAL_TYP((V))->val.off.ref_type)
 #define PVM_VAL_TYP_C_RETURN_TYPE(V) (PVM_VAL_TYP((V))->val.cls.return_type)
 #define PVM_VAL_TYP_C_NARGS(V) (PVM_VAL_TYP((V))->val.cls.nargs)
 #define PVM_VAL_TYP_C_ATYPES(V) (PVM_VAL_TYP((V))->val.cls.atypes)
@@ -446,7 +445,6 @@ enum pvm_type_code
   PVM_TYPE_STRUCT,
   PVM_TYPE_OFFSET,
   PVM_TYPE_CLOSURE,
-  PVM_TYPE_ANY,
   PVM_TYPE_VOID
 };
 
@@ -472,6 +470,7 @@ struct pvm_type
     {
       pvm_val name;
       pvm_val nfields;
+      pvm_val constructor;
       pvm_val *fnames;
       pvm_val *ftypes;
     } sct;
@@ -480,6 +479,7 @@ struct pvm_type
     {
       pvm_val base_type;
       pvm_val unit;
+      pvm_val ref_type;
     } off;
 
     struct
@@ -500,6 +500,7 @@ typedef struct pvm_type *pvm_type;
 #define PVM_VAL_CLS_PROGRAM(V) (PVM_VAL_CLS((V))->program)
 #define PVM_VAL_CLS_ENTRY_POINT(V) (PVM_VAL_CLS((V))->entry_point)
 #define PVM_VAL_CLS_ENV(V) (PVM_VAL_CLS((V))->env)
+#define PVM_VAL_CLS_NAME(V) (PVM_VAL_CLS((V))->name)
 
 /* A struct containing a PVM program.
    Defined elsewhere.  */
@@ -507,6 +508,7 @@ struct pvm_program;
 
 struct pvm_cls
 {
+  pvm_val name;
   struct pvm_program *program;
   pvm_program_program_point entry_point;
   struct pvm_env *env;
@@ -518,9 +520,8 @@ typedef struct pvm_cls *pvm_cls;
 
 #define PVM_VAL_OFF(V) (PVM_VAL_BOX_OFF (PVM_VAL_BOX ((V))))
 
+#define PVM_VAL_OFF_TYPE(V) (PVM_VAL_OFF((V))->type)
 #define PVM_VAL_OFF_MAGNITUDE(V) (PVM_VAL_OFF((V))->magnitude)
-#define PVM_VAL_OFF_UNIT(V) (PVM_VAL_OFF((V))->unit)
-#define PVM_VAL_OFF_BASE_TYPE(V) (PVM_VAL_OFF((V))->base_type)
 
 #define PVM_VAL_OFF_UNIT_BITS 1
 #define PVM_VAL_OFF_UNIT_NIBBLES 4
@@ -542,9 +543,8 @@ typedef struct pvm_cls *pvm_cls;
 
 struct pvm_off
 {
-  pvm_val base_type;
+  pvm_val type;
   pvm_val magnitude;
-  pvm_val unit;
 };
 
 typedef struct pvm_off *pvm_off;
